from TestStack.White.UIItems import Label
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class LabelKeywords(LibraryComponent):
    @keyword
    def verify_label(self, locator, expected):
        """
        Verify text in label
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | actual | expected text | M |
        """
        label = self.state._get_typed_item_by_locator(Label, locator)
        self.state._verify_value(expected, label.Text)



from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword
from TestStack.White.UIItems.WindowItems import Window
from robot.api import logger


class WindowKeywords(LibraryComponent):
    @keyword
    def attach_window(self, window_title):
        self.state.window = self.state.app.GetWindow(window_title)
        logger.console("window {}".format(self.state.window))

    @keyword    
    def select_modal_window(self, locator):
        """
        Select modal window by title
        | Arguments | Usage | (M)andatory / (O)ptional |
        | Title | Modal window title | M |
        """
        self.state.window = self.state.window.ModalWindow(locator)



import clr
import os
dll_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'bin', 'TestStack.White.dll')
clr.AddReference('System')
clr.AddReference(dll_path)
from TestStack.White.UIItems.Finders import SearchCriteria

from robot.api import logger

from WhiteLibrary.keywords import ApplicationKeywords, KeyboardKeywords, WindowKeywords, ScreenshotKeywords
from WhiteLibrary.keywords.items import (ButtonKeywords, 
                            LabelKeywords,
                            ListKeywords, 
                            MenuKeywords,
                            ProgressbarKeywords,
                            SliderKeywords,
                            TabKeywords,
                            TreeKeywords, 
                            TextBoxKeywords,
                            UiItemKeywords)
from WhiteLibrary.keywords.robotlibcore import DynamicCore

STRATEGIES = {"id": "ByAutomationId",
              "text": "ByText",
              "index": "Indexed"}


class WhiteLibrary(DynamicCore):
    ROBOT_LIBRARY_SCOPE = "Global"
    ROBOT_LISTENER_API_VERSION = 2

    def __init__(self):
        self.app = None
        self.window = None
        self.screenshooter = None
        self.ROBOT_LIBRARY_LISTENER = self
        self.screenshot_type = 'desktop'
        self.screenshots_enabled = True
        self.libraries = [ApplicationKeywords(self),
                          ButtonKeywords(self),
                          KeyboardKeywords(self),
                          LabelKeywords(self),
                          ListKeywords(self),
                          MenuKeywords(self),
                          ProgressbarKeywords(self),
                          SliderKeywords(self),
                          TabKeywords(self),
                          TextBoxKeywords(self),
                          TreeKeywords(self),
                          UiItemKeywords(self),
                          WindowKeywords(self),
                          ScreenshotKeywords(self)]
        DynamicCore.__init__(self, self.libraries)
        
    def _get_typed_item_by_locator(self, item_type, locator):
        search_criteria = self._get_search_criteria(locator)
        return self.window.Get[item_type](search_criteria)

    def _get_item_by_locator(self, locator):
        search_criteria = self._get_search_criteria(locator)
        return self.window.Get(search_criteria)

    def _get_search_criteria(self, locator):
        if "=" not in locator:
            locator = "id=" + locator
        search_strategy, locator_value = locator.split("=")
        if search_strategy == "index":
            locator_value = int(locator_value)
        search_method = STRATEGIES[search_strategy]
        method = getattr(SearchCriteria, search_method)
        return method(locator_value)

    def _end_keyword(self, name, attrs):
        if attrs['status'] == 'FAIL':
            if self.screenshot_type == 'desktop' and self.screenshots_enabled:
                self.screenshooter.take_desktop_screenshot()

    def _verify_value(self, expected, actual):
        if expected != actual:
            raise AssertionError("Expected value {}, but found {}".format(expected, actual))



from TestStack.White.UIItems.TabItems import Tab
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class TabKeywords(LibraryComponent):
    @keyword
    def select_tab_page(self, locator, title):
        """ Selects a tab page. """
        tab = self.state._get_typed_item_by_locator(Tab, locator)
        tab.SelectTabPage(title)



from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword
from TestStack.White.InputDevices import Keyboard
from TestStack.White.WindowsAPI import KeyboardInput


SPECIAL_KEYS = ["SHIFT", "CONTROL", "ALT", "LEFT_ALT", "RIGHT_ALT", "RETURN",
                "RIGHT", "BACKSPACE", "LEFT", "ESCAPE", "TAB", "HOME", "END",
                "UP", "DOWN", "INSERT", "DELETE", "CAPS", "F1", "F2", "F3",
                "F4", "F5", "F6", "F7", "F8", "F9", "F11", "F12", "F13", "F14",
                "F15", "F16", "F17", "F18", "F19", "F20", "F21", "F22", "F23",
                "F24", "PAGEUP", "PAGEDOWN", "PRINT", "PRINTSCREEN", "SPACE",
                "NUMLOCK", "SCROLL", "LWIN", "RWIN"]


class KeyboardKeywords(LibraryComponent):
    @keyword    
    def press_special_key(self, key):
        """
        Press a special key (Ctrl, tab, alt for example)
        Accepted special keys are: SHIFT, CONTROL, ALT, LEFT_ALT, RIGHT_ALT,
        RETURN ,RIGHT, BACKSPACE, LEFT, ESCAPE, TAB, HOME, END, UP, DOWN,
        INSERT, DELETE, CAPS, F1, F2, F3, F4, F5, F6, F7, F8, F9, F11, F12,
        F13, F14,F15, F16, F17, F18, F19, F20, F21, F22, F23,F24, PAGEUP,
        PAGEDOWN, PRINT, PRINTSCREEN, SPACE, NUMLOCK, SCROLL, LWIN, RWIN
        Parameters
        ----------
        key - one of the accepted special keys listed above
        """
        if key not in SPECIAL_KEYS:
            raise AttributeError("Allowed special keys are " + str(SPECIAL_KEYS))
        attribute = getattr(KeyboardInput.SpecialKeys, key)
        self.state.window.Keyboard.PressSpecialKey(attribute)

    @keyword
    def hold_special_key(self, key):
        """
        Hold down a special key.
        See keyword Press Special Key for more information about accepted
        special keys.
        Parameters
        ----------
        key - one of the accepted special keys
        """
        if key not in SPECIAL_KEYS:
            raise AttributeError("Allowed special keys are " + str(SPECIAL_KEYS))
        attribute = getattr(KeyboardInput.SpecialKeys, key)
        self.state.window.Keyboard.HoldKey(attribute)

    @keyword
    def leave_special_key(self, key):
        """
        Leave a special key that was previously held down.
        See keyword Press Special Key for more information about accepted
        special keys.
        Parameters
        ----------
        key - one of the accepted special keys
        """
        if key not in SPECIAL_KEYS:
            raise AttributeError("Allowed special keys are " + str(SPECIAL_KEYS))
        attribute = getattr(KeyboardInput.SpecialKeys, key)
        self.state.window.Keyboard.LeaveKey(attribute)

    @keyword
    def press_keys(self, keys):
        """
        Press a key or keys
        Parameters
        ----------
        keys - key or keys to press as string
        """
        self.state.window.Keyboard.Enter(keys)



from TestStack.White.UIItems import ProgressBar
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class ProgressbarKeywords(LibraryComponent):
    @keyword
    def verify_progressbar_value(self, locator, expected):
        """
        Verify progressbar value
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | expected | expected value | M |
        """
        progressbar = self.state._get_typed_item_by_locator(ProgressBar, locator)
        self.state._verify_value(float(expected), progressbar.Value)



from TestStack.White.UIItems import TextBox
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword

class TextBoxKeywords(LibraryComponent):
    @keyword
    def input_text_to_textbox(self, locator, text):
        """
        Write text to textbox
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | text | inserted string | M |
        """
        textBox = self.state._get_typed_item_by_locator(TextBox, locator)
        textBox.Text = text

    @keyword
    def verify_text_in_textbox(self, locator, expected):
        """
        Verify text in textbox
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | actual | expected text | M |
        """
        textbox = self.state._get_typed_item_by_locator(TextBox, locator)
        self.state._verify_value(expected, textbox.Text)



from TestStack.White.UIItems import Slider
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class SliderKeywords(LibraryComponent):
    @keyword
    def set_slider_value(self, locator, value):
        """
        Write slider to value double
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | value | inserted value (must be between scale) | M |
        """
        slider = self.state._get_typed_item_by_locator(Slider, locator)
        slider.Value = float(value)

    @keyword
    def verify_slider_value(self, locator, expected):
        """
        Verify slider value
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | actual | expected double | M |
        """
        slider = self.state._get_typed_item_by_locator(Slider, locator)
        self.state._verify_value(float(expected), slider.Value)



from .buttons import ButtonKeywords
from .label import LabelKeywords
from .listcontrols import ListKeywords
from .menu import MenuKeywords
from .progressbar import ProgressbarKeywords
from .slider import SliderKeywords
from .tab import TabKeywords
from .textbox import TextBoxKeywords
from .tree import TreeKeywords
from .uiitem import UiItemKeywords



import os
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword, PY2
from robot.api import logger
from robot.libraries.BuiltIn import BuiltIn, RobotNotRunningError
from robot.utils import get_link_path

from System.Drawing import Bitmap
from System.Drawing.Imaging import ImageFormat
from TestStack.White import Desktop


class ScreenshotKeywords(LibraryComponent):

    def __init__(self, state):
        super(ScreenshotKeywords, self).__init__(state)
        self.state.screenshooter = self

    @keyword
    def take_desktop_screenshot(self):
        """ Takes a screenshot of the whole desktop and inserts screenshot link to log file.
        Returns
        -------
        string
            path to the screenshot file
        """
        filepath = self._get_screenshot_path("whitelib_screenshot_{index}.png")
        logger.info(get_link_path(filepath, self._log_directory), also_console=True)
        logger.info(
            '</td></tr><tr><td colspan="3">''<a href="{src}"><img src="{src}" width="800px"></a>'.format(
                src=get_link_path(filepath, self._log_directory)), html=True)
        bmp = Desktop.CaptureScreenshot()
        bmp.Save(filepath, ImageFormat.Png)
        return filepath

    @keyword
    def take_screenshots_on_failure(self, status):
        """ Disable or enable automatic screenshot creation on failure.
        Parameters
        ----------
        status: bool or str
            True or False, boolean or string, case insensitive. """
        if (str(status).lower() == 'false'):
            self.state.screenshots_enabled = False
        else:
            self.state.screenshots_enabled = True

    @property
    def _log_directory(self):
        try:
            logfile = BuiltIn().get_variable_value('${LOG FILE}')
            if logfile is None:
                return BuiltIn().get_variable_value('${OUTPUTDIR}')
            return os.path.dirname(logfile)
        except RobotNotRunningError:
            return os.getcwdu() if PY2 else os.getcwd()

    def _get_screenshot_path(self, filename):
        directory = self._log_directory
        filename = filename.replace('/', os.sep)
        index = 0
        while True:
            index += 1
            formatted = filename.format(index=index)
            filepath = os.path.join(directory, formatted)
            # filename didn't contain {index} or unique path was found
            if formatted == filename or not os.path.exists(filepath):
                return filepath


from TestStack.White.UIItems import UIItem
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class UiItemKeywords(LibraryComponent):
    @keyword
    def click_item(self, locator):
        """
        Click an item
        Parameters
        ----------
        locator - element id, text or index prefixed with <locator_type>=
        """
        item = self.state._get_item_by_locator(locator)
        item.Click()
    
    @keyword
    def right_click_item(self, locator):
        """
        Right click an item
        Parameters
        ----------
        locator - element id, text or index prefixed with <locator_type>=
        """
        item = self.state._get_item_by_locator(locator)
        item.RightClick()

    @keyword
    def double_click_item(self, locator):
        """
        Double click an item
        Parameters
        ----------
        locator - element id, text or index prefixed with <locator_type>=
        """
        item = self.state._get_item_by_locator(locator)
        item.DoubleClick()



from TestStack.White.UIItems import Button, CheckBox, RadioButton
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class ButtonKeywords(LibraryComponent):
    @keyword
    def click_button(self, locator):
        """ Clicks button.

        ``locator`` is the locator of the button.
        """
        button = self.state._get_typed_item_by_locator(Button, locator)
        button.Click()

    @keyword
    def button_text_should_be(self, locator, expected_text):
       raise NotImplementedError()

    @keyword
    def button_text_should_contain(self, locator, expected_text):
       raise NotImplementedError()

    @keyword
    def verify_button(self, locator, expected):
        """*DEPRECATED.* Use `Button Text Should Be` or `Button Text Should Contain` instead.

        Verifies text in button.

        ``locator`` is the locator of the button.

        ``expected`` is the expected button text.
        """
        button = self.state._get_typed_item_by_locator(Button, locator)
        self.state._verify_value(expected, button.Text)

    @keyword
    def select_radio_button(self, locator):
        """
        Click button
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        """
        radiobutton = self.state._get_typed_item_by_locator(RadioButton, locator)
        radiobutton.Select()
        
    @keyword
    def verify_radio_button(self, locator, expected):
        """
        Verify value of radio button
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | actual | expected value | M |
        """
        radiobutton = self.state._get_typed_item_by_locator(RadioButton, locator)
        self.state._verify_value(bool(expected), radiobutton.IsSelected)

    @keyword
    def toggle_check_box(self, locator):
        """
        Toggle check box
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        """
        checkbox = self.state._get_typed_item_by_locator(CheckBox, locator)
        checkbox.Toggle()
        
    @keyword
    def verify_check_box(self, locator, expected):
        """
        Verify value of check box
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | actual | expected value | M |
        """
        checkbox = self.state._get_typed_item_by_locator(CheckBox, locator)
        self.state._verify_value(bool(expected), checkbox.IsSelected)



import os
import distutils.sysconfig
from distutils.core import setup
from distutils.extension import Extension

version_data = {}
with open(os.path.join('src', 'WhiteLibrary', 'version.py')) as f:
    exec(f.read(), version_data)

def __path(filename):
    return os.path.join(os.path.dirname(__file__),
                        filename)

build = 0

if os.path.exists(__path('build.info')):
    build = open(__path('build.info')).read().strip()

if version_data['STABLE']:
    version= '{}'.format(version_data['VERSION'])
else: 
    version= '{}.{}.pre'.format(version_data['VERSION'], build)

setup(name         = 'robotframework-whitelibrary',
      version      = version,
      description  = 'Windows GUI testing library for Robot Framework',
      author       = 'SALabs',
      author_email = 'to.be.added@noexist89a887.org',
      url          = 'https://github.com/Omenia/robotframework-whitelibrary',
      install_requires     = ['pythonnet'],
      packages     = ['WhiteLibrary', 'WhiteLibrary.keywords', 'WhiteLibrary.keywords.items'],
      package_dir  = {'WhiteLibrary' : 'src\WhiteLibrary'},
      package_data = {'WhiteLibrary' : ['bin\*.dll']},
      )



from TestStack.White.UIItems.ListBoxItems import ComboBox, ListBox
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword

class ListKeywords(LibraryComponent):
    @keyword
    def select_listbox_value(self, locator, value):
        """
        Select listbox value
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | value | selected value | M |
        """
        listbox = self.state._get_typed_item_by_locator(ListBox, locator)
        listbox.Select(value)

    @keyword    
    def listbox_selection_should_be(self, locator, value):
        """
        Check that the value on the listbox is selected
        Parameters
        ----------
        locator - element id, text or index prefixed with <locator_type>=
        value - value that should be selected
        Raises
        ------
        AssertionError - if the selection was not as expected
        """
        listbox = self.state._get_typed_item_by_locator(ListBox, locator)
        if listbox.SelectedItemText != value:
            raise AssertionError("Expected listbox selection to be " +
                                 value + ", was " + listbox.SelectedItemText)

    @keyword
    def select_combobox_value(self, locator, value):
        """
        Select combobox value
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | value | selected value | M |
        """
        combobox = self.state._get_typed_item_by_locator(ComboBox, locator)
        combobox.Select(value)

    @keyword
    def select_combobox_index(self, locator, index):
        """
        Select combobox index
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | index | combobox index | M |
        """
        combobox = self.state._get_typed_item_by_locator(ComboBox, locator)
        combobox.Select(int(index))

    @keyword
    def verify_combobox_item(self, locator, expected):
        """
        *DEPRECATED* Please use Verify Combobox Selection instead
        Verify combobox selected value
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | actual | expected value | M |
        """
        self.verify_combobox_selection(locator, expected)

    @keyword
    def verify_combobox_selection(self, locator, expected):
        """
        Verify combobox selected value
        | Arguments | Usage | (M)andatory / (O)ptional |
        | locator | element id | M |
        | actual | expected value | M |
        """
        combobox = self.state._get_typed_item_by_locator(ComboBox, locator)
        self.state._verify_value(expected, combobox.EditableText)



from TestStack.White.UIItems.TreeItems import Tree
from WhiteLibrary.keywords.librarycomponent import LibraryComponent
from WhiteLibrary.keywords.robotlibcore import keyword


class TreeKeywords(LibraryComponent):
    @keyword
    def select_tree_node(self, locator, *node_path):
        """ Selects a tree node. """
        tree = self.state._get_typed_item_by_locator(Tree, locator)
        tree.Nodes.GetItem(node_path).Select()
        
    @keyword
    def expand_tree_node(self, locator, *node_path):
        """ Expands a tree node. """
        tree = self.state._get_typed_item_by_locator(Tree, locator)
        tree.Nodes.GetItem(node_path).Expand()
        
    @keyword
    def double_click_tree_node(self, locator, *node_path):
        """ Double-clicks a tree node. """
        tree = self.state._get_typed_item_by_locator(Tree, locator)
        tree.Nodes.GetItem(node_path).DoubleClick()
        
    @keyword
    def right_click_tree_node(self, locator, *node_path):
        """ Right-clicks a tree node. """
        tree = self.state._get_typed_item_by_locator(Tree, locator)
        tree.Nodes.GetItem(node_path).RightClick()
